import weaviate
import json
import logging
from typing import Iterable, List
from internal.embeddings.OpenAIEmbeddingProvider import OpenAIEmbeddingProvider
from internal.embeddings.EmbeddingProvider import EmbeddingProvider
from langchain.schema import Document

class WeaviateClient:
    # Constructor
    def __init__(self, client: weaviate.Client,
                 text_key: str = "content",
                 index_name: str = "documents",
                 embeddingProvider: EmbeddingProvider = OpenAIEmbeddingProvider(),
                 ):
        self.client = client
        self.embeddingProvider = embeddingProvider
        self.text_key = text_key
        self.index_name = index_name

    @classmethod
    def fromUrl(cls, endpoint: str):
        """
        Creates a WeaviateClient from an endpoint
        """
        client = weaviate.Client(endpoint)
        return cls(client)

    def addDocuments(self, documents: Iterable[Document]) -> None:
        """
        Adds a list of documents to the store
        """
        if self.embeddingProvider is None:
            raise Exception("No embedding provider set")

        with self.client.batch as batch:
            for i, text in enumerate(documents):
                data_properties = {self.text_key: text.page_content}
                if text.metadata is not None:
                    for key, val in text.metadata.items():
                        data_properties[key] = val

                vector = self.embeddingProvider.getEmbedding(text.page_content)
                batch.add_data_object(
                        data_object=data_properties,
                        class_name=self.index_name,
                        vector=vector
                    )

    def addDocument(self, document: Document) -> None:
        """
        Adds a given document to the store
        """
        self.addDocuments([document])

    def similaritySearch(self, query: str, k: int = 10) -> List[Document]:
        """
        Searches for similar documents

        Args:
          query: Text to lookup
          k: Number of results to return, default 10
        """
        if self.embeddingProvider is None:
            raise Exception("No embedding provider set")
        vector = self.embeddingProvider.getEmbedding(query)
        return self.similaritySearchByVector(vector, k)

    def similaritySearchByVector(self, vector: List[float], k: int = 10) -> List[Document]:
        """
        Searches for similar documents

        Args:
          vector: Vector to lookup
          k: Number of results to return, default 10
        """
        vectorQuery = { "vector": vector }
        query_obj = self.client.query.get(self.index_name, ["content", "document_name"])
        result = query_obj.with_near_vector(vectorQuery).with_limit(k).do()

        if "errors" in result:
            raise Exception(result["errors"])
        results = []

        print(result)

        for res in result["data"]["Get"][self.index_name]:
            text = res.pop(self.text_key)
            results.append(Document(page_content=text, metadata=res))

        return results

    def removeDocument(self, document_id: str) -> None:
        """
        Removes a document from the store

        Args:
          document: Document to remove
        """
        pass

    def getDocument(self, document_id: str) -> str:
        """
        Returns a document from the store

        Args:
          document: Document to return
        """
        pass


    @property
    def ENDPOINT(self):
        return self.endpoint

    @ENDPOINT.setter
    def ENDPOINT(self, endpoint: str):
        self.endpoint = endpoint

    @property
    def EMBEDDING_PROVIDER(self):
        return self.embeddingProvider

    @EMBEDDING_PROVIDER.setter
    def EMBEDDING_PROVIDER(self, embeddingProvider: EmbeddingProvider):
        self.embeddingProvider = embeddingProvider

    @property
    def TEXT_KEY(self):
        return self.text_key

    @TEXT_KEY.setter
    def TEXT_KEY(self, text_key: str):
        self.text_key = text_key

    @property
    def INDEX_NAME(self):
        return self.index_name

    @INDEX_NAME.setter
    def INDEX_NAME(self, index_name: str):
        self.index_name = index_name




